@@ -39,6 +39,7 @@ class API:  # pylint: disable=too-many-instance-attributes
     def __init__(self, websession: ClientSession = None) -> None:
         """Initialize."""
         self._account_info = {}
+        self._credentials = {}
         self._last_state_update = None  # type: Optional[datetime]
         self._lock = asyncio.Lock()
         self._security_token = None  # type: Optional[str]
@@ -59,17 +60,70 @@ class API:  # pylint: disable=too-many-instance-attributes
             if device.device_json["device_family"] not in NON_COVER_DEVICE_FAMILIES
         }
 
+    async def _send_request(
+        self,
+        method: str,
+        url: str,
+        headers: dict = None,
+        params: dict = None,
+        json: dict = None,
+        login_request: bool = False,
+    ) -> dict:
+
+        attempt = 0
+        while attempt < DEFAULT_REQUEST_RETRIES - 1:
+            if attempt != 0:
+                wait_for = min(2 ** attempt, 5)
+                _LOGGER.warning(f"Device update failed; trying again in {wait_for} seconds")
+                await asyncio.sleep(wait_for)
+
+            attempt += 1
+            data = {}
+            request_headers = headers
+            if not login_request:
+                request_headers["SecurityToken"] = self._security_token
+
+            try:
+                _LOGGER.debug(f"Sending myq api request {url}")
+                async with self._websession.request(
+                        method, url, headers=request_headers, params=params, json=json) as resp:
+                    data = await resp.json(content_type=None)
+                    resp.raise_for_status()
+                    return data
+            except ClientError as err:
+                _LOGGER.debug(f"Attempt {attempt} request failed with exception: {str(err)}")
+                if hasattr(err, "status") and err.status == 401:
+                    if login_request:
+                        self._credentials = {}
+                        raise InvalidCredentialsError("Invalid username/password")
+
+                    _LOGGER.debug(f"Security token has expired, re-authenticating to MyQ")
+                    try:
+                        await self.authenticate(self._credentials.get("username"),
+                                                self._credentials.get("password"),
+                                                True)
+                    except RequestError as auth_err:
+                        message = f"Error trying to re-authenticate to myQ service: {str(auth_err)}"
+                        _LOGGER.error(message)
+                        raise RequestError(message)
+
+                    # Reset the attempt counter as we're now re-authenticated.
+                    attempt = 0
+                    continue
+
+        message = f"Error requesting data from {url}: {data.get('description', str(err))}"
+        _LOGGER.error(message)
+        raise RequestError(message)
+
     async def request(
         self,
         method: str,
         endpoint: str,
-        *,
         headers: dict = None,
         params: dict = None,
         json: dict = None,
         login_request: bool = False,
         api_version: str = BASE_API_VERSION,
-        **kwargs
     ) -> dict:
         """Make a request."""
         api_base = API_BASE.format(api_version)
@@ -77,60 +131,68 @@ class API:  # pylint: disable=too-many-instance-attributes
 
         if not headers:
             headers = {}
-        if not login_request:
-            headers["SecurityToken"] = self._security_token
         headers.update(MYQ_HEADERS)
 
         # The MyQ API can time out if multiple concurrent requests are made, so
-        # ensure that only one gets through at a time:
+        # ensure that only one gets through at a time.
+        # Exception is when this is a login request AND there is already a lock, in that case
+        # we're sending the request anyways as we know there is no active request now.
+        if login_request and self._lock.locked():
+            return await self._send_request(
+                method=method,
+                url=url,
+                headers=headers,
+                params=params,
+                json=json,
+                login_request=login_request,
+            )
+
         async with self._lock:
-            for attempt in (0, DEFAULT_REQUEST_RETRIES):
-                try:
-                    _LOGGER.debug("myq api request {}".format(url))
-                    async with self._websession.request(
-                        method, url, headers=headers, params=params, json=json, **kwargs
-                    ) as resp:
-                        data = await resp.json(content_type=None)
-                        resp.raise_for_status()
-                        return data
-                except ClientError as err:
-                    if "401" in str(err) and login_request:
-                        raise InvalidCredentialsError(
-                            "Invalid username/password"
-                        )
+            return await self._send_request(
+                method=method,
+                url=url,
+                headers=headers,
+                params=params,
+                json=json,
+                login_request=login_request,
+            )
 
-                    if attempt == DEFAULT_REQUEST_RETRIES - 1:
-                        raise RequestError(
-                            "Error requesting data from {0}: {1}".format(
-                                url, data.get("description", str(err))
-                            )
-                        )
 
-                    wait_for = min(2 ** attempt, 5)
-
-                    _LOGGER.warning(
-                        "Device update failed; trying again in %s seconds", wait_for
-                    )
-
-                    await asyncio.sleep(wait_for)
-
-    async def authenticate(self, username: str, password: str) -> None:
+    async def authenticate(self, username: str, password: str, token_only = False) -> None:
         """Authenticate and get a security token."""
+        if username is None or password is None:
+            _LOGGER.debug("No username/password, most likely due to previous failed authentication.")
+            return
+
+        self._credentials = {"username": username, "password": password}
         # Retrieve and store the initial security token:
+        _LOGGER.debug("Sending authentication request to MyQ")
         auth_resp = await self.request(
-            "post",
-            "Login",
+            method="post",
+            endpoint="Login",
             json={"Username": username, "Password": password},
             login_request=True,
         )
-        self._security_token = auth_resp["SecurityToken"]
+        self._security_token = auth_resp.get("SecurityToken")
+        if self._security_token is None:
+            _LOGGER.debug("No security token received.")
+            raise RequestError(
+                "Authentication response did not contain a security token yet one is expected."
+            )
+
+        if token_only:
+            return
 
         # Retrieve and store account info:
+        _LOGGER.debug("Retrieving account information")
         self._account_info = await self.request(
-            "get", "My", params={"expand": "account"}
+            method="get",
+            endpoint="My",
+            params={"expand": "account"}
         )
 
         # Retrieve and store initial set of devices:
+        _LOGGER.debug("Retrieving MyQ information")
         await self.update_device_info()
 
     async def update_device_info(self) -> dict:
@@ -147,12 +209,14 @@ class API:  # pylint: disable=too-many-instance-attributes
             return
 
         devices = []
+        _LOGGER.debug("Retrieving accounts")
         accounts_resp = await self.request(
             "get", "Accounts"
         )
         if accounts_resp is not None and accounts_resp.get("Items") is not None:
             for account in accounts_resp["Items"]:
                 account_id = account["Id"]
+                _LOGGER.debug(f"Retrieving devices for account {account_id}")
                 devices_resp = await self.request(
                     "get", "Accounts/{0}/Devices".format(account_id), api_version=DEVICES_API_VERSION
                 )
@@ -160,6 +224,7 @@ class API:  # pylint: disable=too-many-instance-attributes
                     devices += devices_resp["items"]
 
         if not devices:
+            _LOGGER.debug("Retrieving device information")
             devices_resp = await self.request(
                 "get", "Accounts/{0}/Devices".format(self.account_id), api_version=DEVICES_API_VERSION
             )
@@ -191,5 +256,5 @@ class API:  # pylint: disable=too-many-instance-attributes
 async def login(username: str, password: str, websession: ClientSession = None) -> API:
     """Log in to the API."""
     api = API(websession)
-    await api.authenticate(username, password)
+    await api.authenticate(username, password, False)
     return api
