@@ -70,7 +70,7 @@ class MyQDevice:
     @property
     def online(self) -> bool:
         """Return whether the device is online."""
-        return self.device_json.get("online") is True
+        return self.device_json["state"].get("online") is True
 
     @property
     def open_allowed(self) -> bool:
@@ -113,9 +113,10 @@ class MyQDevice:
             infos = re.search(rule, self.href)
             if infos is not None:
                 account_id = infos.group(1)
+        _LOGGER.debug(f"Sending command {state_comand} for {self.name}")
         await self._api.request(
-            "put",
-            "Accounts/{0}/Devices/{1}/actions".format(
+            method="put",
+            endpoint="Accounts/{0}/Devices/{1}/actions".format(
                 account_id, self.device_id
             ),
             json={"action_type": state_command},
